@@ -3,21 +3,18 @@ use strict;
 use warnings;
 use Carp;
 
-our $VERSION = '0.0402';
-
-our $ac_opt = {};
-our $ac_define_class = {};
+our $VERSION = '0.0601';
 
 sub import{
-    my $caller_class = caller;
+    my $caller_pkg = caller;
     
     {
         # export function
         no strict 'refs';
-        *{ "${caller_class}::ac" } = \&Simo::ac;
+        *{ "${caller_pkg}::ac" } = \&Simo::ac;
         
         # caller inherit Simo
-        push @{ "${caller_class}::ISA" }, __PACKAGE__;
+        push @{ "${caller_pkg}::ISA" }, __PACKAGE__;
     }
 
     # auto strict and warnings
@@ -27,74 +24,124 @@ sub import{
 
 sub new{
     my ( $proto, @args ) = @_;
+
+    # bless
+    my $self = {};
+    my $pkg = ref $proto || $proto;
+    bless $self, $pkg;
     
     # check args
     @args = %{ $args[0] } if ref $args[0] eq 'HASH';
-    croak 'key-value pairs must be passed to new method' if @args % 2;
-    
-    # bless
-    my $self = {};
-    my $class = ref $proto || $proto;
-    bless $self, $class;
+    croak 'key-value pairs must be passed to new' if @args % 2;
     
     # set args
     while( my ( $attr, $val ) = splice( @args, 0, 2 ) ){
-        croak "Invalid key '$attr' is passed to ${class}::new" unless $self->can( $attr );
+        croak "Invalid key '$attr' is passed to new" unless $self->can( $attr );
         no strict 'refs';
         $self->$attr( $val );
     }
     return $self;
 }
 
-# Accessor register
-our %valid_ac_opt = map{ $_ => 1 } qw( default constrain filter trigger set_hook get_hook hash_force );
+# get value specify attr names
+sub get_attrs{
+    my ( $self, @attrs ) = @_;
+    
+    @attrs = @{ $attrs[0] } if ref $attrs[0] eq 'ARRAY';
+    
+    my @vals;
+    foreach my $attr ( @attrs ){
+        croak "Invalid key '$attr' is passed to get_attrs" unless $self->can( $attr );
+        my $val = $self->$attr;
+        push @vals, $val;
+    }
+    wantarray ? @vals : $vals[0];
+}
 
+# get value as hash specify attr names
+sub get_attrs_as_hash{
+    my ( $self, @attrs ) = @_;
+    my @vals = $self->get_attrs( @attrs );
+    
+    my %attrs;
+    @attrs{ @attrs } = @vals;
+    
+    wantarray ? %attrs : \%attrs;
+}
+
+# set values
+sub set_attrs{
+    my ( $self, @args ) = @_;
+
+    # check args
+    @args = %{ $args[0] } if ref $args[0] eq 'HASH';
+    croak 'key-value pairs must be passed to set_attrs' if @args % 2;
+    
+    # set args
+    while( my ( $attr, $val ) = splice( @args, 0, 2 ) ){
+        croak "Invalid key '$attr' is passed to set_attrs" unless $self->can( $attr );
+        no strict 'refs';
+        $self->$attr( $val );
+    }
+    return $self;
+}
+
+
+# create accessor
 sub ac(@){
-
-    # accessor info
-    my ( $self, $attr, $ac_define_class, @vals ) = _SIMO_get_ac_info();
-    
-    # check accessor info
-    my $class = ref $self;
-    
-    # check and rearrange accessor option;
-    my $ac_opt = {};
-    
-    $ac_opt->{ default } = shift if @_ % 2;
-    my $hook_options_exist = {};
-    
-    while( my( $key, $val ) = splice( @_, 0, 2 ) ){
-        croak "$key of ${ac_define_class}::$attr is invalid accessor option" unless $valid_ac_opt{ $key };
-        carp "${ac_define_class}::$attr : $@" unless _SIMO_check_hook_options_order( $key, $hook_options_exist );
-        
-        $ac_opt->{ $key } = $val;
-    }
-    
-    # register accessor option
-    $Simo::ac_opt{ $ac_define_class }{ $attr } = $ac_opt;
-
-    # redefine real acessor
-    my $ac_redefine = qq/sub ${ac_define_class}::$attr { _SIMO_ac_real( '$attr' , \@_ ) }/;
-    
-    {
-        no warnings 'redefine';
-        eval $ac_redefine;
-    }
+    # Simo process
+    my ( $self, $attr, @vals ) = _SIMO_process( @_ );
     
     # call accessor
     $self->$attr( @vals );
 }
 
+# accessor option
+my %VALID_AC_OPT = map{ $_ => 1 } qw( default constrain filter trigger set_hook get_hook hash_force read_only );
+
+# Simo process. register accessor option and create accessor.
+sub _SIMO_process{
+    # accessor info
+    my ( $self, $attr, $pkg, @vals ) = _SIMO_get_ac_info();
+    
+    # check and rearrange accessor option;
+    my $ac_opt = {};
+    
+    $ac_opt->{ default } = shift if @_ % 2; 
+        # ( Unnamed default option is is now not recommended. this will be deleted in future 2019/01/01 )
+    
+    my $hook_options_exist = {};
+    
+    while( my( $key, $val ) = splice( @_, 0, 2 ) ){
+        croak "$key of ${pkg}::$attr is invalid accessor option" 
+            unless $VALID_AC_OPT{ $key };
+        
+        carp "${pkg}::$attr : $@" 
+            unless _SIMO_check_hook_options_order( $key, $hook_options_exist );
+        
+        $ac_opt->{ $key } = $val;
+    }
+    
+
+    # create accessor
+    {
+        my $code = _SIMO_create_accessor( $pkg, $attr, $ac_opt );
+        no warnings qw( redefine closure );
+        eval"sub ${pkg}::${attr} $code";
+    }
+    return ( $self, $attr, @vals );
+}
+
 # check hook option order ( constrain, filter, and trigger )
-our %valid_hook_options = ( constrain => 1, filter => 2, trigger => 3 );
+my %VALID_HOOK_OPT = ( constrain => 1, filter => 2, trigger => 3 );
 
 sub _SIMO_check_hook_options_order{
     my ( $key, $hook_options_exist ) = @_;
     
-    return 1 unless $valid_hook_options{ $key };
+    return 1 unless $VALID_HOOK_OPT{ $key };
     
     foreach my $hook_option_exist ( keys %{ $hook_options_exist } ){
-        if( $valid_hook_options{ $key } < $valid_hook_options{ $hook_option_exist } ){
+        if( $VALID_HOOK_OPT{ $key } < $VALID_HOOK_OPT{ $hook_option_exist } ){
             $@ = "$key option should be appear before $hook_option_exist option";
             return 0;
         }
@@ -103,116 +150,143 @@ sub _SIMO_check_hook_options_order{
     return 1;
 }
 
-# Real accessor.
-sub _SIMO_ac_real{
-    my ( $attr, $self, @vals ) = @_;
+# create accessor.
+sub _SIMO_create_accessor{
+    my ( $pkg, $attr, $ac_opt ) = @_;
     
-    # check args
-    my $class = ref $self;
-    croak "$attr must be called from object." unless $class;
+    my $e =
+        qq/{\n/ .
+        # arg recieve
+        qq/    my ( \$self, \@vals ) = \@_;\n\n/;
     
-    # get accessor defined class
-    $Simo::ac_define_class->{ $class }{ $attr } ||= _SIMO_get_ac_define_class( $class, $attr );
-    my $ac_define_class = $Simo::ac_define_class->{ $class }{ $attr };
+    if( defined $ac_opt->{ default } ){
+        # default value
+        $e .=
+        qq/    if( ! exists( \$self->{ $attr } ) ){\n/ .
+        qq/        \$self->{ $attr } = \$ac_opt->{ default };\n/ .
+        qq/    }\n/ .
+        qq/    \n/;
+    }
     
-    # get accessor option
-    my $ac_opt = $Simo::ac_opt{ $ac_define_class }{ $attr };
-    
-    # init by default value
-    $self->{ $attr } = $ac_opt->{ default } unless exists $self->{ $attr };
-    
-    # return value( return old_value in case setter is called )
-    my $ret = $self->{ $attr };
-    
-    # set value if value is defined
-    if( @vals ){
-        # rearrange value;
-        my $val = @vals == 1 ? $vals[0] :
-                  @vals >= 2 && $ac_opt->{ hash_force } ? { @vals } :
-                  @vals >= 2 ? [ @vals ] :
-                  undef;
-    
-        # setter hook function
-        # ( set_hook option is now not recommended. this option will be deleted in future 2019 )
-        if( $ac_opt->{ set_hook } ){
-            eval{ $val = $ac_opt->{ set_hook }->($self,$val) };
-            confess $@ if $@;
-        }
-        
-        # constrain
-        if( my $constrains = $ac_opt->{ constrain } ){
-            $constrains = [ $constrains ] unless ref $constrains eq 'ARRAY';
-            foreach my $constrain ( @{ $constrains } ){
-                croak "constrain of ${ac_define_class}::$attr must be code ref"
-                    unless ref $constrain eq 'CODE';
-                    
-                local $_ = $val;
-                my $ret = $constrain->( $val );
-                croak "Illegal value $val is passed to ${ac_define_class}::$attr"
-                    unless $ret;
-            }
-        }
-        
-        # filter
-        if( my $filters = $ac_opt->{ filter } ){
-            $filters = [ $filters ] unless ref $filters eq 'ARRAY';
-            foreach my $filter ( @{ $filters } ){
-                croak "filter of ${ac_define_class}::$attr must be code ref"
-                    unless ref $filter eq 'CODE';
-                                    
-                local $_ = $val;
-                $val = $filter->( $val );
-            }
-        }
-        
-        # set new value
-        $self->{ $attr } = $val;
-        
-        # trigger
-        if( my $triggers = $ac_opt->{ trigger } ){
-            $triggers = [ $triggers ] unless ref $triggers eq 'ARRAY';
-            foreach my $trigger ( @{ $triggers } ){
-                croak "trigger of ${ac_define_class}::$attr must be code ref"
-                    unless ref $trigger eq 'CODE';
+    # get value
+    $e .=
+        qq/    my \$ret = \$self->{ $attr };\n\n/;
 
-                local $_ = $self;
-                $trigger->( $self );
-            }
-        }
+    if ( $ac_opt->{ read_only } ){
+        $e .=
+        qq/    Carp::croak( "${pkg}::$attr is read only" ) if \@vals;\n\n/;
+        
+        goto END_OF_VALUE_SETTING;
     }
-    else{
-        # getter hook function
-        # ( get_hook option is now not recommended. this option will be deleted in future 2019 )
-        if( $ac_opt->{ get_hook } ){
-            eval{ $ret = $ac_opt->{ get_hook }->($self, $ret) };
-            confess $@ if $@;
-        }
+        
+    $e .=
+        qq/    if( \@vals ){\n/ .
+    
+    # rearrange value
+        qq/        my \$val = \@vals == 1 ? \$vals[0] :\n/;
+    $e .= $ac_opt->{ hash_force } ?
+        qq/                  \@vals >= 2 ? { \@vals } :\n/ :
+        qq/                  \@vals >= 2 ? [ \@vals ] :\n/;
+    $e .=
+        qq/                  undef;\n\n/;
+    
+    if( defined $ac_opt->{ set_hook } ){
+        # set_hook option
+        #( set_hook option is is now not recommended. this option will be deleted in future 2019/01/01 )
+        $e .=
+        qq/        eval{ \$val = \$ac_opt->{ set_hook }->( \$self, \$val ) };\n/ .
+        qq/        Carp::confess( \$@ ) if \$@;\n\n/;
     }
-    return $ret;
-}
+    
+    if( defined $ac_opt->{ constrain } ){
+        # constrain option
 
-# Get accessor define class
-sub _SIMO_get_ac_define_class{
-    my ( $class, $attr ) = @_;
-    
-    my $ac_define_class = ( caller 2 )[ 3 ];
-    
-    if( $ac_define_class =~ /^(.+)::/ ){
-        $ac_define_class = $1;
+        $ac_opt->{ constrain } = [ $ac_opt->{ constrain } ] 
+            unless ref $ac_opt->{ constrain } eq 'ARRAY';
+        
+        foreach my $constrain ( @{ $ac_opt->{ constrain } } ){
+            Carp::croak( "constrain of ${pkg}::$attr must be code ref" )
+                unless ref $constrain eq 'CODE';
+        }
+        
+        $e .=
+        qq/        foreach my \$constrain ( \@{ \$ac_opt->{ constrain } } ){\n/ .
+        qq/            local \$_ = \$val;\n/ .
+        qq/            my \$ret = \$constrain->( \$val );\n/ .
+        qq/            Carp::croak( "Illegal value \$val is passed to ${pkg}::$attr" )\n/ .
+        qq/                unless \$ret;\n/ .
+        qq/        }\n\n/;
     }
-    return $ac_define_class;
+    
+    if( defined $ac_opt->{ filter } ){
+        # filter option
+        $ac_opt->{ filter } = [ $ac_opt->{ filter } ] 
+            unless ref $ac_opt->{ filter } eq 'ARRAY';
+        
+        foreach my $filter ( @{ $ac_opt->{ filter } } ){
+            Carp::croak( "filter of ${pkg}::$attr must be code ref" )
+                unless ref $filter eq 'CODE';
+        }
+        
+        $e .=
+        qq/        foreach my \$filter ( \@{ \$ac_opt->{ filter } } ){\n/ .
+        qq/            local \$_ = \$val;\n/ .
+        qq/            \$val = \$filter->( \$val );\n/ .
+        qq/        }\n\n/;
+    }
+    
+    # set value
+    $e .=
+        qq/        \$self->{ $attr } = \$val;\n\n/;
+    
+    if( defined $ac_opt->{ trigger } ){
+        $ac_opt->{ trigger } = [ $ac_opt->{ trigger } ]
+            unless ref $ac_opt->{ trigger } eq 'ARRAY';
+        
+        foreach my $trigger ( @{ $ac_opt->{ trigger } } ){
+            Carp::croak( "trigger of ${pkg}::$attr must be code ref" )
+                unless ref $trigger eq 'CODE';
+        }
+        
+        # trigger option
+        $e .=
+        qq/        foreach my \$trigger ( \@{ \$ac_opt->{ trigger } } ){\n/ .
+        qq/            local \$_ = \$self;\n/ .
+        qq/            \$trigger->( \$self );\n/ .
+        qq/        }\n/;
+    }
+    
+    $e .=
+        qq/    }\n/;
+    
+    END_OF_VALUE_SETTING:
+    
+    if( defined $ac_opt->{ get_hook } ){
+        # get_hook option
+        # ( get_hook option is is now not recommended. this option will be deleted in future 2019/01/01 )
+        $e .=
+        qq/    eval{ \$ret = \$ac_opt->{ get_hook }->( \$self, \$ret ) };\n/ .
+        qq/    Carp::confess( \$@ ) if \$@;\n/;
+    }
+    
+    #return
+    $e .=
+        qq/    return \$ret;\n/ .
+        qq/}\n/;
+    
+    return $e;
 }
 
 # Helper to get acsessor info;
 sub _SIMO_get_ac_info {
     package DB;
-    my @caller = caller 2;
+    my @caller = caller 3;
     
     my ( $self, @vals ) = @DB::args;
     my $sub = $caller[ 3 ];
-    my ( $ac_define_class, $attr ) = $sub =~ /^(.*)::(.+)$/;
+    my ( $pkg, $attr ) = $sub =~ /^(.*)::(.+)$/;
 
-    return ( $self, $attr, $ac_define_class, @vals );
+    return ( $self, $attr, $pkg, @vals );
 }
 
 =head1 NAME
@@ -221,7 +295,7 @@ Simo - Very simple framework for Object Oriented Perl.
 
 =head1 VERSION
 
-Version 0.0402
+Version 0.0601
 
 =cut
 
@@ -276,7 +350,12 @@ writing new and accessors repeatedly.
         my $date = substr( $self->issue_datetime, 0, 10 );
         $self->issue_date( $date );
     }
+    
+    # read only accessor
+    sub get_size{ ac default => 5, read_only => 1 }
+    
     1;
+    
 =cut
 
 =head2 Using class and accessors
@@ -303,6 +382,9 @@ writing new and accessors repeatedly.
     # trigger( issue_date is updated '2009-01-01' )
     $book->issue_datetime( '2009-01-01 12:33:45' );
     my $issue_date = $book->issue_date;
+    
+    # read only accessor
+    $book->get_size;
 
 =cut
 
@@ -431,6 +513,14 @@ and you can define more than one trigger.
 
 =cut
 
+=head3 read_only option
+
+Read only accessor is defined
+
+    sub get_size{ ac default => 5, read_only => 1 }
+
+Accessor name should be contain 'get_'.If not, warnings is happen.
+
 =head3 hash_force option
 
 If you pass array to accessor, Normally list convert to array ref.
@@ -449,18 +539,6 @@ If you use hash_force option, you convert list to hash ref
 
 =cut
 
-=head3 set_hook option
-
-set_hook option is now not recommended. this option will be deleted in future 2019/01/01
-
-=cut
-
-=head3 get_hook option
-
-get_hook option is now not recommended. this option will be deleted in future 2019/01/01
-
-=cut
-
 =head2 Order of constrain, filter and trigger
 
 =over 4
@@ -498,14 +576,31 @@ You can get old value when you use accessor as setter.
 
 ac is exported. This is used by define accessor. 
 
-=cut
+=head1 METHOD
 
 =head2 new
 
 orveridable new method.
 
-=cut
+=head2 get_attrs
 
+    my( $title, $author ) = $book->get_attrs( 'title', 'author' );
+
+=head2  get_attrs_as_hash
+
+    my %hash = $book->get_attrs( 'title', 'author' );
+
+or
+
+    my $hash_ref = $book->get_attrs( 'title', 'author' );
+
+=head2 set_attrs
+
+    $book->set_attrs( title => 'Simple OO', author => 'kimoto' );
+
+return value is $self. so method chaine is available
+
+    $book->set_attrs( title => 'Simple OO', author => 'kimoto' )->some_method;
 
 =head1 MORE TECHNIQUES
 
